Package: stringfish
Title: Alt String Implementation
Version: 0.19.2
Date: 2026-08-04
Authors@R: c(
    person("Travers", "Ching", email = "traversc@gmail.com", role = c("aut", "cre", "cph")),
    person("Phillip", "Hazel", role = c("ctb"), comment = "Bundled PCRE2 code"),
    person("Zoltan", "Herczeg", role = c("ctb", "cph"), comment = "Bundled PCRE2 code"),
    person("University of Cambridge", role = c("cph"), comment = "Bundled PCRE2 code"),
    person("Tilera Corporation", role = c("cph"), comment = "Stack-less Just-In-Time compiler bundled with PCRE2"),
    person("Yann", "Collet", role = c("ctb", "cph"), comment = "Yann Collet is the author of the bundled xxHash code"),
    person("Martin", "Leitner-Ankerl", role = c("ctb", "cph"), comment = "Bundled ankerl::unordered_dense code"))
Maintainer: Travers Ching <traversc@gmail.com>
Description: Provides an extendable, performant and multithreaded 'alt-string' implementation backed by 'C++' vectors and strings.
License: GPL-3
Biarch: true
Encoding: UTF-8
Depends: R (>= 3.6.0)
SystemRequirements: GNU make, C++17
LinkingTo: Rcpp (>= 0.12.18.3), RcppParallel (>= 6.1.1)
Imports: Rcpp, RcppParallel
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Copyright: Copyright for the bundled 'PCRE2' library is held by
        University of Cambridge, Zoltan Herczeg and Tilera Corporation
        (Stack-less Just-In-Time compiler); Copyright for the bundled
        'xxHash' code is held by Yann Collet; Copyright for the bundled
        'ankerl::unordered_dense' code is held by Martin
        Leitner-Ankerl.
URL: https://github.com/traversc/stringfish
BugReports: https://github.com/traversc/stringfish/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-08-04 08:23:40 UTC; ted
Author: Travers Ching [aut, cre, cph],
  Phillip Hazel [ctb] (Bundled PCRE2 code),
  Zoltan Herczeg [ctb, cph] (Bundled PCRE2 code),
  University of Cambridge [cph] (Bundled PCRE2 code),
  Tilera Corporation [cph] (Stack-less Just-In-Time compiler bundled with
    PCRE2),
  Yann Collet [ctb, cph] (Yann Collet is the author of the bundled xxHash
    code),
  Martin Leitner-Ankerl [ctb, cph] (Bundled ankerl::unordered_dense code)
Repository: CRAN
Date/Publication: 2026-08-04 10:30:17 UTC
Built: R 4.6.1; aarch64-apple-darwin23; 2026-08-04 13:21:48 UTC; unix
Archs: stringfish.so.dSYM
